// RemoteXY select connection mode and include library 
#define REMOTEXY_MODE__ESP32CORE_BLE


#include <Arduino.h>
#include "BasicStepperDriver.h"

// Motor steps per revolution. Most steppers are 200 steps or 1.8 degrees/step
#define MOTOR_STEPS 200
#define RPM 50

// Since microstepping is set externally, make sure this matches the selected mode
// If it doesn't, the motor will move at a different RPM than chosen
// 1=full step, 2=half step etc.
#define MICROSTEPS 16

// All the wires needed for full functionality
#define DIR 6
#define STEP 7
//Uncomment line to use enable/disable functionality
#define SLEEP 4

stepper_28byj48 stepper_tildado = stepper_28byj48(8, 9, 10, 11, TILDADO, 2);
 //stepper_28byj48 stepper_persiana = stepper_28byj48(9, 13, 10, 12, PERSIANA, 1, 11, 8);

// 2-wire basic config, microstepping is hardwired on the driver
BasicStepperDriver stepper(MOTOR_STEPS, DIR, STEP);

//Uncomment line to use enable/disable functionality
// BasicStepperDriver stepper(MOTOR_STEPS, DIR, STEP, SLEEP);

#include <BLEDevice.h>


// RemoteXY connection settings 
#define REMOTEXY_BLUETOOTH_NAME "RemoteXY"


#include <RemoteXY.h>

// RemoteXY GUI configuration  
#pragma pack(push, 1)  
uint8_t RemoteXY_CONF[] =   // 84 bytes
  { 255,5,0,0,0,77,0,17,0,0,0,31,1,106,200,1,1,5,0,1,
  29,87,24,24,3,31,24,83,117,98,105,114,0,1,59,87,24,24,3,31,
  24,66,97,106,97,114,0,1,28,129,24,24,3,31,24,43,0,1,58,129,
  24,24,3,31,24,45,0,2,33,31,44,22,0,31,16,24,24,79,78,0,
  79,70,70,0 };
  
// this structure defines all the variables and events of your control interface 
struct {

    // input variables
  uint8_t subir_persiana; // =1 if button pressed, else =0
  uint8_t bajar_persiana; // =1 if button pressed, else =0
  uint8_t aumentar_tildado; // =1 if button pressed, else =0
  uint8_t disminuir_tildado; // =1 if button pressed, else =0
  uint8_t operacion_manual; // =1 if switch ON and =0 if OFF

    // other variable
  uint8_t connect_flag;  // =1 if wire connected, else =0

} RemoteXY;   
#pragma pack(pop)

#include "stepper_tildado.h"
#include "sensor_luz.h"
#include "BasicStepperDriver.h"

#define MOTOR_STEPS 200
#define RPM 120
#define MICROSTEPS 16
#define DIR 12
#define STEP 13

#define DIFERENCIA_NIVELES_ALTURA 666

#define PIN_FIN_CARRERA 14

TaskHandle_t Tildado;
TaskHandle_t Persiana;

int nivel_actual= 0;
int target_level = 0;
BasicStepperDriver stepper(MOTOR_STEPS, DIR, STEP);

int en_calibracion_inicial = 1;
int en_calibracion = 1;

stepper_tildado motor_tildado = stepper_tildado(5, 18, 19, 21, 2);
sensor_luz sensor_luz_ldr = sensor_luz(15, 25, 26, 27);

void mover_altura(){
  if ((target_level-nivel_actual) && target_level==4){
    while (!digitalRead(PIN_FIN_CARRERA)) {
      stepper.rotate(50);
    }
    nivel_actual = 4;
  } else {
    int grados = (target_level - nivel_actual)*DIFERENCIA_NIVELES_ALTURA;
    stepper.rotate(grados);
    nivel_actual = target_level;
  }
}

void setup() {
  Serial.begin(115200);
  RemoteXY_Init (); 
  Serial.println("Startingggg...");
  stepper.begin(RPM, MICROSTEPS);
  pinMode(PIN_FIN_CARRERA, INPUT_PULLDOWN);


     stepper.begin(RPM, MICROSTEPS);
    // if using enable/disable on ENABLE pin (active LOW) instead of SLEEP uncomment next line
    stepper.setEnableActiveState(LOW);
    // stepper.enable();
    stepper.rotate(3*(360));
    delay(5000);
    // stepper.disable();

  stepper_tildado.stepper_vueltas(4, UP);
  //stepper_persiana.stepper_vueltas(2, DOWN);

  xTaskCreatePinnedToCore(
                    TildadoCode,   /* Task function. */
                    "Tildado",     /* name of task. */
                    10000,       /* Stack size of task */
                    NULL,        /* parameter of the task */
                    1,           /* priority of the task */
                    &Tildado,      /* Task handle to keep track of created task */
                    0);          /* pin task to core 0 */                  
  delay(500); 

  //create a task that will be executed in the Task2code() function, with priority 1 and executed on core 1
  xTaskCreatePinnedToCore(
                    PersianaCode,   /* Task function. */
                    "Persiana",     /* name of task. */
                    10000,       /* Stack size of task */
                    NULL,        /* parameter of the task */
                    2,           /* priority of the task */
                    &Persiana,      /* Task handle to keep track of created task */
                    1);          /* pin task to core 1 */
  delay(500);
}

void PersianaCode(void * pvParameters) {
  Serial.print("Persiana running on core ");
  Serial.println(xPortGetCoreID());

  for(;;){
    RemoteXY_Handler ();
    sensor_luz_ldr.read_ldr();
    if (en_calibracion_inicial) {
    stepper.setRPM(30);
      while (!digitalRead(PIN_FIN_CARRERA)) {
        stepper.rotate(50);
        Serial.print(".");
      }
      stepper.setRPM(60);
      Serial.println("FIN CALIBRACION");
      nivel_actual = 4;
      en_calibracion_inicial = 0;
    } else {
      if (RemoteXY.operacion_manual) {
        if (RemoteXY.subir_persiana) {
          if (nivel_actual < 4) target_level = nivel_actual + 1;
        } else if (RemoteXY.bajar_persiana) {
          if (nivel_actual > 0) target_level = nivel_actual - 1;
        }
      } else {
        if (RemoteXY.aumentar_tildado) {
          en_calibracion = 1;
          Serial.println("en calibracion");
        }
        if (en_calibracion) {
          if (RemoteXY.subir_persiana) {
            while (!digitalRead(PIN_FIN_CARRERA)) {
              stepper.rotate(50);
            }
            nivel_actual = 4;
            en_calibracion = 0;
            Serial.println("Saliendo de calibracion");
          }
        } else {
          target_level = sensor_luz_ldr.get_level_altura();
        }
      }
      mover_altura();
      RemoteXY_delay(500);
    } 

  }
}

void TildadoCode(void * pvParameters) {
  Serial.print("Tildado running on core ");
  Serial.println(xPortGetCoreID());

  for(;;){
    if (RemoteXY.operacion_manual) {
      if (RemoteXY.aumentar_tildado) {
        if (motor_tildado.get_nivel() < 2) {
          motor_tildado.stepper_conv(motor_tildado.get_nivel() + 1);
        }
      } else if (RemoteXY.disminuir_tildado) {
        if (motor_tildado.get_nivel() > 0){
          motor_tildado.stepper_conv(motor_tildado.get_nivel() - 1);
        }
      }
    } else {
      motor_tildado.stepper_conv(sensor_luz_ldr.get_level_tildado());
    }
    delay(100);
  } 

}

void loop() {

    
}
